/*	$NetBSD$	*/

/*-
 * Copyright (c) 2014 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/cdefs.h>
#include <sys/types.h>

#ifdef _KERNEL
__KERNEL_RCSID(0, "$NetBSD: in.c,v 1.148 2014/09/09 20:16:12 rmind Exp $");
#include <sys/systm.h>
#else
__RCSID("$NetBSD: in.c,v 1.148 2014/09/09 20:16:12 rmind Exp $");
#include <stdio.h>
#include <stdarg.h>
static const uint8_t hexdigits[] = "0123456789abcdef";
#endif
#include <net/if_dl.h>

int
dl_print(char *buf, size_t len, const void *av, size_t al)
{
	const uint8_t *ap = av;
	char *cp = buf;
	char *ecp = buf + len;

#define ADDC(c) do { \
		if (cp >= ecp) {\
			cp++; \
		} else \
			*cp++ = (char)(c); \
	} while (/*CONSTCOND*/0)

#define ADDX(v) do { \
		uint8_t n = hexdigits[(v)]; \
		ADDC(n); \
	} while (/*CONSTCOND*/0)

	for (size_t i = 0; i < al; i++) {
		ADDX((u_int)ap[i] >> 4);
		ADDX(ap[i] & 0xf);
		ADDC(':');
	}
	if (cp > buf)
		--cp;
	if (ecp > buf) {
		if (cp < ecp)
			*cp = '\0';
		else
			*--ecp = '\0';
	}
	return (int)(cp - buf);
}

int
sdl_snprintf(char *buf, size_t len, const void *v, const char *fmt, ...)
{
	const struct sockaddr_dl *sdl = v;
	char abuf[LINK_ADDRSTRLEN];
	char pbuf[256];
	va_list ap;

	va_start(ap, fmt); 
	vsnprintf(pbuf, sizeof(pbuf), fmt, ap);
	va_end(ap);

	dl_print(abuf, sizeof(abuf), sdl->sdl_data,
	    (size_t)(sdl->sdl_nlen + sdl->sdl_alen));

	return snprintf(buf, len, "%stype=%d index=%d alen=%d "
	    "nlen=%d slen=%d data=%s", pbuf, sdl->sdl_type, sdl->sdl_index,
	    sdl->sdl_alen, sdl->sdl_nlen, sdl->sdl_slen, abuf);
}
