/*	$NetBSD$	*/

/*-
 * Copyright (c) 2014 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Christos Zoulas.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/cdefs.h>
__RCSID("$NetBSD$");

#include "net/dl_print.c"

#include <atf-c.h>

static const struct {
	uint8_t data[20];
	size_t dlen;
	const char *str;
	int len;
} tst[] = {
	{
		{ 0x01, 0xa2, 0x03, 0xc4, 0x05, 0xf6 },
		6,
		"01:a2:03:c4:05:f6",
		17,
	},
	{
		{ 0x00, 0x00 },
		2,
		"00:00",
		5,
	},
};


ATF_TC(dl_print);
ATF_TC_HEAD(dl_print, tc)
{

	atf_tc_set_md_var(tc, "descr", "printing of link address");
}

ATF_TC_BODY(dl_print, tc)
{
	char buf[LINK_ADDRSTRLEN];
	int r;
	size_t l = sizeof(buf);

	for (size_t i = 0; i < __arraycount(tst); i++) {
		r = dl_print(buf, l, tst[i].data, tst[i].dlen);
		ATF_REQUIRE_STREQ(buf, tst[i].str);
		ATF_REQUIRE_EQ(r, tst[i].len);
	}

	l = 4;
	for (size_t i = 0; i < __arraycount(tst); i++) {
		r = dl_print(buf, l, tst[i].data, tst[i].dlen);
		ATF_CHECK(strncmp(buf, tst[i].str, l - 1) == 0);
		if (r > (int)l)
			ATF_REQUIRE_EQ(buf[l - 1], '\0');
		ATF_REQUIRE_EQ(r, tst[i].len);
	}
}

ATF_TC(sdl_snprintf);
ATF_TC_HEAD(sdl_snprintf, tc)
{

	atf_tc_set_md_var(tc, "descr", "printing of sockaddr_dl");
}

static int 
prt(char *buf, size_t len, const struct sockaddr_dl *sdl)
{
	return snprintf(buf, len, "type=%d index=%d alen=%d "
	      "nlen=%d slen=%d", sdl->sdl_type, sdl->sdl_index,
	      sdl->sdl_alen, sdl->sdl_nlen, sdl->sdl_slen);
}

ATF_TC_BODY(sdl_snprintf, tc)
{
	char buf[1024];
	char res[1024];
	char part[1024];
	int r, e;
	size_t l = sizeof(buf);
	struct sockaddr_dl sdl;

	memset(&sdl, 0, sizeof(sdl));
	for (size_t i = 0; i < __arraycount(tst); i++) {
		memcpy(sdl.sdl_data, tst[i].data, tst[i].dlen);
		sdl.sdl_nlen = (uint8_t)tst[i].dlen;
		sdl.sdl_slen = 0;
		r = sdl_snprintf(buf, l, &sdl, "%zu: ", i);
		prt(part, sizeof(part), &sdl);
		e = snprintf(res, l, "%zu: %s data=%s", i, part, tst[i].str);
		ATF_REQUIRE_STREQ(buf, res);
		ATF_REQUIRE_EQ(r, e);
	}

	l = 8;
	for (size_t i = 0; i < __arraycount(tst); i++) {
		memcpy(sdl.sdl_data, tst[i].data, tst[i].dlen);
		sdl.sdl_nlen = (uint8_t)tst[i].dlen;
		sdl.sdl_slen = 0;
		r = sdl_snprintf(buf, l, &sdl, "%zu: ", i);
		prt(part, sizeof(part), &sdl);
		e = snprintf(res, l, "%zu: %s data=%s", i, part, tst[i].str);

		ATF_REQUIRE_STREQ(buf, res);
		ATF_REQUIRE_EQ(r, e);
	}
}

ATF_TP_ADD_TCS(tp)
{

	ATF_TP_ADD_TC(tp, dl_print);
	ATF_TP_ADD_TC(tp, sdl_snprintf);
	return atf_no_error();
}
